# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/runner/work/ImagingS/ImagingS/src/ImagingS/Gui/uic/NewDocumentDialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NewDocumentDialog(object):
    def setupUi(self, NewDocumentDialog):
        NewDocumentDialog.setObjectName("NewDocumentDialog")
        NewDocumentDialog.resize(286, 142)
        self.verticalLayout = QtWidgets.QVBoxLayout(NewDocumentDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(NewDocumentDialog)
        self.widget.setObjectName("widget")
        self.formLayout = QtWidgets.QFormLayout(self.widget)
        self.formLayout.setObjectName("formLayout")
        self.lblWidth = QtWidgets.QLabel(self.widget)
        self.lblWidth.setObjectName("lblWidth")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblWidth)
        self.sbxWidth = QtWidgets.QSpinBox(self.widget)
        self.sbxWidth.setMinimum(100)
        self.sbxWidth.setMaximum(3000)
        self.sbxWidth.setSingleStep(50)
        self.sbxWidth.setProperty("value", 600)
        self.sbxWidth.setObjectName("sbxWidth")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.sbxWidth)
        self.lblHeight = QtWidgets.QLabel(self.widget)
        self.lblHeight.setObjectName("lblHeight")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lblHeight)
        self.sbxHeight = QtWidgets.QSpinBox(self.widget)
        self.sbxHeight.setMinimum(100)
        self.sbxHeight.setMaximum(3000)
        self.sbxHeight.setSingleStep(50)
        self.sbxHeight.setProperty("value", 600)
        self.sbxHeight.setObjectName("sbxHeight")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.sbxHeight)
        self.verticalLayout.addWidget(self.widget)
        self.buttonBox = QtWidgets.QDialogButtonBox(NewDocumentDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(NewDocumentDialog)
        self.buttonBox.accepted.connect(NewDocumentDialog.accept)
        self.buttonBox.rejected.connect(NewDocumentDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(NewDocumentDialog)

    def retranslateUi(self, NewDocumentDialog):
        _translate = QtCore.QCoreApplication.translate
        NewDocumentDialog.setWindowTitle(_translate("NewDocumentDialog", "New Document"))
        self.lblWidth.setText(_translate("NewDocumentDialog", "Width"))
        self.lblHeight.setText(_translate("NewDocumentDialog", "Height"))
