# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/runner/work/ImagingS/ImagingS/src/ImagingS/Gui/uic/CodePage.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CodePage(object):
    def setupUi(self, CodePage):
        CodePage.setObjectName("CodePage")
        CodePage.resize(400, 300)
        self.horizontalLayout = QtWidgets.QHBoxLayout(CodePage)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tlbWindow = QtWidgets.QToolBar(CodePage)
        self.tlbWindow.setOrientation(QtCore.Qt.Vertical)
        self.tlbWindow.setObjectName("tlbWindow")
        self.horizontalLayout.addWidget(self.tlbWindow)
        self.widMain = QtWidgets.QWidget(CodePage)
        self.widMain.setObjectName("widMain")
        self.grdCode = QtWidgets.QGridLayout(self.widMain)
        self.grdCode.setContentsMargins(0, 0, 0, 0)
        self.grdCode.setObjectName("grdCode")
        self.swgMain = QtWidgets.QStackedWidget(self.widMain)
        self.swgMain.setObjectName("swgMain")
        self.pagCode = QtWidgets.QWidget()
        self.pagCode.setObjectName("pagCode")
        self.grdCode1 = QtWidgets.QGridLayout(self.pagCode)
        self.grdCode1.setContentsMargins(0, 0, 0, 0)
        self.grdCode1.setObjectName("grdCode1")
        self.tetCode = QtWidgets.QTextEdit(self.pagCode)
        self.tetCode.setObjectName("tetCode")
        self.grdCode1.addWidget(self.tetCode, 0, 0, 1, 1)
        self.swgMain.addWidget(self.pagCode)
        self.pagTree = QtWidgets.QWidget()
        self.pagTree.setObjectName("pagTree")
        self.grdCode2 = QtWidgets.QGridLayout(self.pagTree)
        self.grdCode2.setContentsMargins(0, 0, 0, 0)
        self.grdCode2.setObjectName("grdCode2")
        self.trvTree = QtWidgets.QTreeView(self.pagTree)
        self.trvTree.setObjectName("trvTree")
        self.grdCode2.addWidget(self.trvTree, 0, 0, 1, 1)
        self.swgMain.addWidget(self.pagTree)
        self.grdCode.addWidget(self.swgMain, 0, 0, 1, 1)
        self.horizontalLayout.addWidget(self.widMain)
        self.actRestore = QtWidgets.QAction(CodePage)
        self.actRestore.setObjectName("actRestore")
        self.actCommit = QtWidgets.QAction(CodePage)
        self.actCommit.setObjectName("actCommit")
        self.actSwitch = QtWidgets.QAction(CodePage)
        self.actSwitch.setObjectName("actSwitch")
        self.tlbWindow.addAction(self.actSwitch)
        self.tlbWindow.addAction(self.actRestore)
        self.tlbWindow.addAction(self.actCommit)

        self.retranslateUi(CodePage)
        self.swgMain.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(CodePage)

    def retranslateUi(self, CodePage):
        _translate = QtCore.QCoreApplication.translate
        CodePage.setWindowTitle(_translate("CodePage", "Code"))
        self.tlbWindow.setWindowTitle(_translate("CodePage", "Code Tool Box"))
        self.actRestore.setText(_translate("CodePage", "Restore"))
        self.actCommit.setText(_translate("CodePage", "Commit"))
        self.actSwitch.setText(_translate("CodePage", "Switch"))
