# Generated by UI compiler

from ._ui_NewDocumentDialog import Ui_NewDocumentDialog as NewDocumentDialog
from ._ui_VisualPage import Ui_VisualPage as VisualPage
from ._ui_DocumentEditor import Ui_DocumentEditor as DocumentEditor
from ._ui_CodePage import Ui_CodePage as CodePage
from ._ui_MainWindow import Ui_MainWindow as MainWindow


__all__ = (
    "NewDocumentDialog",
    "VisualPage",
    "DocumentEditor",
    "CodePage",
    "MainWindow",
)
