# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/runner/work/ImagingS/ImagingS/src/ImagingS/Gui/uic/MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1028, 717)
        MainWindow.setDockNestingEnabled(True)
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AllowNestedDocks|QtWidgets.QMainWindow.AllowTabbedDocks|QtWidgets.QMainWindow.AnimatedDocks)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        MainWindow.setCentralWidget(self.centralwidget)
        self.mnbWindow = QtWidgets.QMenuBar(MainWindow)
        self.mnbWindow.setGeometry(QtCore.QRect(0, 0, 1028, 26))
        self.mnbWindow.setObjectName("mnbWindow")
        self.mnuFile = QtWidgets.QMenu(self.mnbWindow)
        self.mnuFile.setObjectName("mnuFile")
        self.mnuEdit = QtWidgets.QMenu(self.mnbWindow)
        self.mnuEdit.setObjectName("mnuEdit")
        self.mnuView = QtWidgets.QMenu(self.mnbWindow)
        self.mnuView.setObjectName("mnuView")
        self.mnuBrush = QtWidgets.QMenu(self.mnbWindow)
        self.mnuBrush.setObjectName("mnuBrush")
        self.mnuTool = QtWidgets.QMenu(self.mnbWindow)
        self.mnuTool.setObjectName("mnuTool")
        self.mnuTransform = QtWidgets.QMenu(self.mnbWindow)
        self.mnuTransform.setObjectName("mnuTransform")
        self.mnuDrawing = QtWidgets.QMenu(self.mnbWindow)
        self.mnuDrawing.setObjectName("mnuDrawing")
        MainWindow.setMenuBar(self.mnbWindow)
        self.stbMain = QtWidgets.QStatusBar(MainWindow)
        self.stbMain.setObjectName("stbMain")
        MainWindow.setStatusBar(self.stbMain)
        self.dwgDrawings = QtWidgets.QDockWidget(MainWindow)
        self.dwgDrawings.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dwgDrawings.setObjectName("dwgDrawings")
        self.wgDrawings = QtWidgets.QWidget()
        self.wgDrawings.setObjectName("wgDrawings")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.wgDrawings)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.trvDrawings = QtWidgets.QTreeView(self.wgDrawings)
        self.trvDrawings.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.trvDrawings.setObjectName("trvDrawings")
        self.verticalLayout.addWidget(self.trvDrawings)
        self.dwgDrawings.setWidget(self.wgDrawings)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dwgDrawings)
        self.dwgBrushes = QtWidgets.QDockWidget(MainWindow)
        self.dwgBrushes.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dwgBrushes.setObjectName("dwgBrushes")
        self.wgBrushes = QtWidgets.QWidget()
        self.wgBrushes.setObjectName("wgBrushes")
        self.verticalLayout1 = QtWidgets.QVBoxLayout(self.wgBrushes)
        self.verticalLayout1.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout1.setSpacing(0)
        self.verticalLayout1.setObjectName("verticalLayout1")
        self.trvBrushes = QtWidgets.QListView(self.wgBrushes)
        self.trvBrushes.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.trvBrushes.setObjectName("trvBrushes")
        self.verticalLayout1.addWidget(self.trvBrushes)
        self.dwgBrushes.setWidget(self.wgBrushes)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dwgBrushes)
        self.dwgTransforms = QtWidgets.QDockWidget(MainWindow)
        self.dwgTransforms.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dwgTransforms.setObjectName("dwgTransforms")
        self.wgTransforms = QtWidgets.QWidget()
        self.wgTransforms.setObjectName("wgTransforms")
        self.verticalLayout2 = QtWidgets.QVBoxLayout(self.wgTransforms)
        self.verticalLayout2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout2.setSpacing(0)
        self.verticalLayout2.setObjectName("verticalLayout2")
        self.trvTransforms = QtWidgets.QTreeView(self.wgTransforms)
        self.trvTransforms.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.trvTransforms.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.trvTransforms.setObjectName("trvTransforms")
        self.verticalLayout2.addWidget(self.trvTransforms)
        self.dwgTransforms.setWidget(self.wgTransforms)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dwgTransforms)
        self.dwgProperties = QtWidgets.QDockWidget(MainWindow)
        self.dwgProperties.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dwgProperties.setObjectName("dwgProperties")
        self.wgProperties = QtWidgets.QWidget()
        self.wgProperties.setObjectName("wgProperties")
        self.verticalLayout3 = QtWidgets.QVBoxLayout(self.wgProperties)
        self.verticalLayout3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout3.setSpacing(0)
        self.verticalLayout3.setObjectName("verticalLayout3")
        self.trvProperties = QtWidgets.QTreeView(self.wgProperties)
        self.trvProperties.setObjectName("trvProperties")
        self.verticalLayout3.addWidget(self.trvProperties)
        self.dwgProperties.setWidget(self.wgProperties)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dwgProperties)
        self.actNew = QtWidgets.QAction(MainWindow)
        self.actNew.setObjectName("actNew")
        self.actOpen = QtWidgets.QAction(MainWindow)
        self.actOpen.setObjectName("actOpen")
        self.actSave = QtWidgets.QAction(MainWindow)
        self.actSave.setObjectName("actSave")
        self.actSaveAs = QtWidgets.QAction(MainWindow)
        self.actSaveAs.setObjectName("actSaveAs")
        self.actExport = QtWidgets.QAction(MainWindow)
        self.actExport.setObjectName("actExport")
        self.actClose = QtWidgets.QAction(MainWindow)
        self.actClose.setObjectName("actClose")
        self.actAbout = QtWidgets.QAction(MainWindow)
        self.actAbout.setObjectName("actAbout")
        self.actQuit = QtWidgets.QAction(MainWindow)
        self.actQuit.setObjectName("actQuit")
        self.actUndo = QtWidgets.QAction(MainWindow)
        self.actUndo.setObjectName("actUndo")
        self.actRedo = QtWidgets.QAction(MainWindow)
        self.actRedo.setObjectName("actRedo")
        self.actDrawingRemove = QtWidgets.QAction(MainWindow)
        self.actDrawingRemove.setObjectName("actDrawingRemove")
        self.actDrawingClear = QtWidgets.QAction(MainWindow)
        self.actDrawingClear.setObjectName("actDrawingClear")
        self.actBrushSolid = QtWidgets.QAction(MainWindow)
        self.actBrushSolid.setObjectName("actBrushSolid")
        self.actBrushRemove = QtWidgets.QAction(MainWindow)
        self.actBrushRemove.setObjectName("actBrushRemove")
        self.actBrushClear = QtWidgets.QAction(MainWindow)
        self.actBrushClear.setObjectName("actBrushClear")
        self.actTransformRemove = QtWidgets.QAction(MainWindow)
        self.actTransformRemove.setObjectName("actTransformRemove")
        self.actTransformClear = QtWidgets.QAction(MainWindow)
        self.actTransformClear.setObjectName("actTransformClear")
        self.actViewVisual = QtWidgets.QAction(MainWindow)
        self.actViewVisual.setCheckable(True)
        self.actViewVisual.setObjectName("actViewVisual")
        self.actViewCode = QtWidgets.QAction(MainWindow)
        self.actViewCode.setCheckable(True)
        self.actViewCode.setObjectName("actViewCode")
        self.actDrawingsRefresh = QtWidgets.QAction(MainWindow)
        self.actDrawingsRefresh.setObjectName("actDrawingsRefresh")
        self.actTransformsRefresh = QtWidgets.QAction(MainWindow)
        self.actTransformsRefresh.setObjectName("actTransformsRefresh")
        self.actBrushesRefresh = QtWidgets.QAction(MainWindow)
        self.actBrushesRefresh.setObjectName("actBrushesRefresh")
        self.actPropertiesRefresh = QtWidgets.QAction(MainWindow)
        self.actPropertiesRefresh.setObjectName("actPropertiesRefresh")
        self.mnuFile.addAction(self.actNew)
        self.mnuFile.addAction(self.actOpen)
        self.mnuFile.addSeparator()
        self.mnuFile.addAction(self.actSave)
        self.mnuFile.addAction(self.actSaveAs)
        self.mnuFile.addSeparator()
        self.mnuFile.addAction(self.actExport)
        self.mnuFile.addSeparator()
        self.mnuFile.addAction(self.actClose)
        self.mnuFile.addSeparator()
        self.mnuFile.addAction(self.actAbout)
        self.mnuFile.addAction(self.actQuit)
        self.mnuView.addAction(self.actViewVisual)
        self.mnuView.addAction(self.actViewCode)
        self.mnuView.addSeparator()
        self.mnuBrush.addAction(self.actBrushSolid)
        self.mnbWindow.addAction(self.mnuFile.menuAction())
        self.mnbWindow.addAction(self.mnuEdit.menuAction())
        self.mnbWindow.addAction(self.mnuView.menuAction())
        self.mnbWindow.addAction(self.mnuDrawing.menuAction())
        self.mnbWindow.addAction(self.mnuTransform.menuAction())
        self.mnbWindow.addAction(self.mnuBrush.menuAction())
        self.mnbWindow.addAction(self.mnuTool.menuAction())
        self.trvDrawings.addAction(self.actDrawingRemove)
        self.trvDrawings.addAction(self.actDrawingClear)
        self.trvBrushes.addAction(self.actBrushRemove)
        self.trvBrushes.addAction(self.actBrushClear)
        self.trvTransforms.addAction(self.actTransformRemove)
        self.trvTransforms.addAction(self.actTransformClear)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "ImagingS"))
        self.mnuFile.setTitle(_translate("MainWindow", "&File"))
        self.mnuEdit.setTitle(_translate("MainWindow", "&Edit"))
        self.mnuView.setTitle(_translate("MainWindow", "&View"))
        self.mnuBrush.setTitle(_translate("MainWindow", "&Brush"))
        self.mnuTool.setTitle(_translate("MainWindow", "Tool"))
        self.mnuTransform.setTitle(_translate("MainWindow", "&Transform"))
        self.mnuDrawing.setTitle(_translate("MainWindow", "&Drawing"))
        self.dwgDrawings.setWindowTitle(_translate("MainWindow", "Drawings"))
        self.dwgBrushes.setWindowTitle(_translate("MainWindow", "Brushes"))
        self.dwgTransforms.setWindowTitle(_translate("MainWindow", "Transforms"))
        self.dwgProperties.setWindowTitle(_translate("MainWindow", "Properties"))
        self.actNew.setText(_translate("MainWindow", "&New"))
        self.actNew.setShortcut(_translate("MainWindow", "Ctrl+N"))
        self.actOpen.setText(_translate("MainWindow", "&Open"))
        self.actOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actSave.setText(_translate("MainWindow", "&Save"))
        self.actSave.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.actSaveAs.setText(_translate("MainWindow", "Save As"))
        self.actSaveAs.setShortcut(_translate("MainWindow", "Ctrl+Shift+S"))
        self.actExport.setText(_translate("MainWindow", "&Export"))
        self.actClose.setText(_translate("MainWindow", "&Close"))
        self.actClose.setShortcut(_translate("MainWindow", "Ctrl+F4"))
        self.actAbout.setText(_translate("MainWindow", "About"))
        self.actQuit.setText(_translate("MainWindow", "&Quit"))
        self.actUndo.setText(_translate("MainWindow", "Undo"))
        self.actUndo.setShortcut(_translate("MainWindow", "Ctrl+Z"))
        self.actRedo.setText(_translate("MainWindow", "Redo"))
        self.actRedo.setShortcut(_translate("MainWindow", "Ctrl+Y"))
        self.actDrawingRemove.setText(_translate("MainWindow", "Remove"))
        self.actDrawingClear.setText(_translate("MainWindow", "Clear"))
        self.actBrushSolid.setText(_translate("MainWindow", "Solid"))
        self.actBrushRemove.setText(_translate("MainWindow", "Remove"))
        self.actBrushClear.setText(_translate("MainWindow", "Clear"))
        self.actTransformRemove.setText(_translate("MainWindow", "Remove"))
        self.actTransformClear.setText(_translate("MainWindow", "Clear"))
        self.actViewVisual.setText(_translate("MainWindow", "Visual"))
        self.actViewVisual.setShortcut(_translate("MainWindow", "Ctrl+Shift+V"))
        self.actViewCode.setText(_translate("MainWindow", "Code"))
        self.actViewCode.setShortcut(_translate("MainWindow", "Ctrl+Shift+C"))
        self.actDrawingsRefresh.setText(_translate("MainWindow", "Refresh"))
        self.actTransformsRefresh.setText(_translate("MainWindow", "Refresh"))
        self.actBrushesRefresh.setText(_translate("MainWindow", "Refresh"))
        self.actPropertiesRefresh.setText(_translate("MainWindow", "Refresh"))
