# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/runner/work/ImagingS/ImagingS/src/ImagingS/Gui/uic/VisualPage.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VisualPage(object):
    def setupUi(self, VisualPage):
        VisualPage.setObjectName("VisualPage")
        VisualPage.resize(530, 300)
        self.horizontalLayout = QtWidgets.QHBoxLayout(VisualPage)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tlbMain = QtWidgets.QToolBar(VisualPage)
        self.tlbMain.setOrientation(QtCore.Qt.Vertical)
        self.tlbMain.setObjectName("tlbMain")
        self.horizontalLayout.addWidget(self.tlbMain)
        self.widMain = QtWidgets.QWidget(VisualPage)
        self.widMain.setObjectName("widMain")
        self.grdMain = QtWidgets.QGridLayout(self.widMain)
        self.grdMain.setContentsMargins(0, 0, 0, 0)
        self.grdMain.setObjectName("grdMain")
        self.horizontalLayout.addWidget(self.widMain)
        self.actDrawingLine = QtWidgets.QAction(VisualPage)
        self.actDrawingLine.setObjectName("actDrawingLine")
        self.actDrawingCurve = QtWidgets.QAction(VisualPage)
        self.actDrawingCurve.setObjectName("actDrawingCurve")
        self.actDrawingEllipse = QtWidgets.QAction(VisualPage)
        self.actDrawingEllipse.setObjectName("actDrawingEllipse")
        self.actTransformTranslate = QtWidgets.QAction(VisualPage)
        self.actTransformTranslate.setObjectName("actTransformTranslate")
        self.actTransformScale = QtWidgets.QAction(VisualPage)
        self.actTransformScale.setObjectName("actTransformScale")
        self.actTransformRotate = QtWidgets.QAction(VisualPage)
        self.actTransformRotate.setObjectName("actTransformRotate")
        self.actTransformSkew = QtWidgets.QAction(VisualPage)
        self.actTransformSkew.setObjectName("actTransformSkew")
        self.actTransformMatrix = QtWidgets.QAction(VisualPage)
        self.actTransformMatrix.setObjectName("actTransformMatrix")
        self.actTransformGroup = QtWidgets.QAction(VisualPage)
        self.actTransformGroup.setObjectName("actTransformGroup")
        self.actDrawingPolygon = QtWidgets.QAction(VisualPage)
        self.actDrawingPolygon.setObjectName("actDrawingPolygon")
        self.actDrawingPolyline = QtWidgets.QAction(VisualPage)
        self.actDrawingPolyline.setObjectName("actDrawingPolyline")
        self.actDrawingRectangle = QtWidgets.QAction(VisualPage)
        self.actDrawingRectangle.setObjectName("actDrawingRectangle")
        self.actClip = QtWidgets.QAction(VisualPage)
        self.actClip.setObjectName("actClip")
        self.tlbMain.addAction(self.actDrawingLine)
        self.tlbMain.addAction(self.actDrawingPolyline)
        self.tlbMain.addAction(self.actDrawingPolygon)
        self.tlbMain.addAction(self.actDrawingRectangle)
        self.tlbMain.addAction(self.actDrawingCurve)
        self.tlbMain.addAction(self.actDrawingEllipse)
        self.tlbMain.addSeparator()
        self.tlbMain.addAction(self.actTransformTranslate)
        self.tlbMain.addAction(self.actTransformScale)
        self.tlbMain.addAction(self.actTransformRotate)
        self.tlbMain.addAction(self.actTransformSkew)
        self.tlbMain.addAction(self.actTransformMatrix)
        self.tlbMain.addAction(self.actTransformGroup)
        self.tlbMain.addSeparator()
        self.tlbMain.addAction(self.actClip)

        self.retranslateUi(VisualPage)
        QtCore.QMetaObject.connectSlotsByName(VisualPage)

    def retranslateUi(self, VisualPage):
        _translate = QtCore.QCoreApplication.translate
        VisualPage.setWindowTitle(_translate("VisualPage", "Visual"))
        self.tlbMain.setWindowTitle(_translate("VisualPage", "Visual Tool Box"))
        self.actDrawingLine.setText(_translate("VisualPage", "Line"))
        self.actDrawingCurve.setText(_translate("VisualPage", "Curve"))
        self.actDrawingEllipse.setText(_translate("VisualPage", "Ellipse"))
        self.actTransformTranslate.setText(_translate("VisualPage", "Translate"))
        self.actTransformScale.setText(_translate("VisualPage", "Scale"))
        self.actTransformRotate.setText(_translate("VisualPage", "Rotate"))
        self.actTransformSkew.setText(_translate("VisualPage", "Skew"))
        self.actTransformMatrix.setText(_translate("VisualPage", "Matrix"))
        self.actTransformGroup.setText(_translate("VisualPage", "Group"))
        self.actDrawingPolygon.setText(_translate("VisualPage", "Polygon"))
        self.actDrawingPolyline.setText(_translate("VisualPage", "Polyline"))
        self.actDrawingRectangle.setText(_translate("VisualPage", "Rectangle"))
        self.actClip.setText(_translate("VisualPage", "Clip"))
