# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/runner/work/ImagingS/ImagingS/src/ImagingS/Gui/uic/DocumentEditor.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DocumentEditor(object):
    def setupUi(self, DocumentEditor):
        DocumentEditor.setObjectName("DocumentEditor")
        DocumentEditor.resize(400, 300)
        self.gridLayout = QtWidgets.QGridLayout(DocumentEditor)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.swgMain = QtWidgets.QStackedWidget(DocumentEditor)
        self.swgMain.setObjectName("swgMain")
        self.pagVisual = QtWidgets.QWidget()
        self.pagVisual.setGeometry(QtCore.QRect(0, 0, 400, 254))
        self.pagVisual.setObjectName("pagVisual")
        self.grdVisual = QtWidgets.QGridLayout(self.pagVisual)
        self.grdVisual.setContentsMargins(0, 0, 0, 0)
        self.grdVisual.setObjectName("grdVisual")
        self.swgMain.addWidget(self.pagVisual)
        self.pagCode = QtWidgets.QWidget()
        self.pagCode.setGeometry(QtCore.QRect(0, 0, 400, 254))
        self.pagCode.setObjectName("pagCode")
        self.grdCode = QtWidgets.QGridLayout(self.pagCode)
        self.grdCode.setContentsMargins(0, 0, 0, 0)
        self.grdCode.setObjectName("grdCode")
        self.swgMain.addWidget(self.pagCode)
        self.gridLayout.addWidget(self.swgMain, 0, 0, 1, 1)

        self.retranslateUi(DocumentEditor)
        self.swgMain.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DocumentEditor)

    def retranslateUi(self, DocumentEditor):
        _translate = QtCore.QCoreApplication.translate
        DocumentEditor.setWindowTitle(_translate("DocumentEditor", "Editor"))
